#!/bin/bash
echo -ne "
-------------------------------------------------------------------------
                PTMLab - Arch Linux Installer v2024.11.26
-------------------------------------------------------------------------

"
# Configure the install medium's pacman to use parallel downloads
echo "Turning on parallel downloads"
sed -i 's/^#ParallelDownloads/ParallelDownloads/' /etc/pacman.conf
# Run reflector so UK based pacman mirrors are prioritised
echo "Running reflector to use UK based Pacman mirrors"
reflector -c 'United Kingdom' -a 12 -p https --sort rate --save /etc/pacman.d/mirrorlist
echo -ne "
-------------------------------------------------------------------------
                            1) Partitioning
-------------------------------------------------------------------------

"
# Run lsblk, filtering out anything other than partitions usable for the install
lsblk | grep --color=never "sd[a-z]"
# Set the EFI partition
read -p 'Please enter the drive value for the EFI partition: /dev/' efipar
# Offer to wipe the EFI partition
read -p "Do you wish to wipe the EFI partition? (y/n): " efiwipe
# Set the root partition
read -p 'Please enter the drive value for the root partition: /dev/' rootpar
# Set an optional /home partition
read -p '(Optional) Please enter the drive value for the home partition: /dev/' homepar
# If set, offer to wipe the home partition
if [[ -n "${homepar}"  ]] then
	read -p "Do you wish to wipe the home partition? (y/n): " homewipe
fi
# Set an optional partition for the pacman cache
# If set, this can be used for pacstrap
read -p "(Optional) Do you wish to set a partition for the Pacman cache? (y/n): " cacheyesno
if [[ "${cacheyesno}" == "y" ]] then
	read -p "Is this a new cache partition? (y/n): " cachewipe
fi
if [[ "${cacheyesno}" == "y" ]] then
	read -p '(Optional) Please enter the drive value for the cache partition: /dev/' cachepar
fi
# Set an optional partition for game installs
read -p "(Optional) Do you wish to set a partition for installing games? (y/n): " gameparyesno
if [[ "${gameparyesno}" == "y" ]] then
	read -p "Is this a new partition? (y/n): " gameparwipe
fi
if [[ "${cacheyesno}" == "y" ]] then
	read -p '(Optional) Please enter the drive value for the game partition: /dev/' gamepar
fi
echo -ne "
-------------------------------------------------------------------------
                            2) User Setup
-------------------------------------------------------------------------

"
# Set the username for the initial account
read -p "Please enter a username: " username
# Set a password for the account and only continue if password validates
while true
do
	read -rsp "Please enter a password: " password
	echo -ne "\n"
	read -rsp "Please re-enter the password: " passwordconfirm
    echo -ne "\n"
	# Validate the password
	if [[ "$password" == "$passwordconfirm" ]] then
    	echo "Credentials saved"
		break
	else
		echo -ne "Passwords do not match. \n"
	fi
done
echo -ne "
-------------------------------------------------------------------------
                            3) Before we begin
-------------------------------------------------------------------------

Just to confirm, you've selected the following:

"
# Show a list of all chosen partition options and the name of the user to be created
echo "EFI Partition: /dev/"$efipar
echo "Root Partition: /dev/"$rootpar
# Check if a home partition is being used, if so show the location
if [[ -n "${homepar}"  ]] then
    echo "Home Partition: /dev/"$homepar
else
	echo "Not using a home partition"
fi
# Check if a cache partition is being used, if so show the location
if [[ -n "${cachepar}"  ]] then
    echo "Cache Partition: /dev/"$cachepar
	if [[ "${cachewipe}" == "y" ]] then
		echo "This is a new cache partition"
	else
		echo "This is a pre-existing cache partition"
	fi
else
	echo -ne "Not using/setting up a Pacman cache\n"
fi
# Check if a games partition is being used, if so show the location
if [[ -n "${gamepar}"  ]] then
    echo "Games Partition: /dev/"$gamepar
	if [[ "${cachewipe}" == "y" ]] then
		echo "This is a new games partition"
	else
		echo "This is a pre-existing games partition"
	fi
else
	echo -ne "Not using/setting up a games partition\n"
fi
# Show the initial user account name
echo "User:" $username
# Ask to confirm installation
read -rep "Do you wish to proceed? (y/n) " proceed
if [[ "${proceed}" == "y" ]] then
    # Unmount the intended root partition if, for any reason, it's already mounted
    umount -l /mnt
    # Format the intended root partition and mount it
    yes | mkfs.ext4 /dev/$rootpar
    mount /dev/$rootpar /mnt
    # If using a cache partition, mount it now
	if [[ -n "${cachepar}"  ]] then
		if [[ "${cachewipe}" == "y" ]] then
			yes | mkfs.ext4 /dev/$cachepar
		fi
    	mount /dev/$cachepar /var/cache/pacman/pkg
	fi
    # Run pacstrap
	if [[ -n "${cachepar}"  ]] then
    	pacstrap -c /mnt base base-devel mkinitcpio iptables-nft reflector networkmanager grub efibootmgr dosfstools os-prober nano --needed --noconfirm
	else
		pacstrap -i /mnt base base-devel mkinitcpio iptables-nft reflector networkmanager grub efibootmgr dosfstools os-prober nano --needed --noconfirm
	fi
    # Wipe the intended EFI partition if it was requested
    if [[ "${efiwipe}" == "y" ]] then
        umount -l /mnt/efi
        mkfs.fat -F32 /dev/$efipar
    fi
    # Mount the EFI partition
    mkdir /mnt/efi
    mount /dev/$efipar /mnt/efi
    # Wipe the intended home partition if it was requested
    if [[ "${homewipe}" == "y" ]] then
        yes | mkfs.ext4 /dev/$homepar
    fi
    # If it was set, mount the home partition
    if [[ -n "${homepar}" ]] then
        mount /dev/$homepar /mnt/home
    fi
    # If it was set, unmount the cache partition and remount it to the installation
    if [[ -n "${cachepar}" ]] then
		umount -l /var/cache/pacman/pkg
        mount /dev/$cachepar /mnt/var/cache/pacman/pkg
    fi
    # Wipe the intended games partition if it was requested
    if [[ "${gameparwipe}" == "y" ]] then
        yes | mkfs.ext4 /dev/$gamepar
    fi
    # If it was set, mount the games partition
    if [[ -n "${gamepar}" ]] then
        mkdir /mnt/games
        mount /dev/$gamepar /mnt/games
    fi
    # Generate the fstab file
    genfstab -U /mnt >> /mnt/etc/fstab
    # Enter the fresh installation
    arch-chroot /mnt /bin/bash <<EOF
    echo "Setting up Pacman"
    # Amend pacman.conf to preference
    sed -i 's/^#Color/Color\nILoveCandy/' /etc/pacman.conf
    sed -i 's/^#ParallelDownloads/ParallelDownloads/' /etc/pacman.conf
    sed -i "/\[multilib\]/,/Include/"'s/^#//' /etc/pacman.conf
    # Run reflector so UK based pacman mirrors are prioritised
    reflector -c 'United Kingdom' -a 12 -p https --sort rate --save /etc/pacman.d/mirrorlist
    # Set up locale and time zone for United Kingdom
    # US English is also enabled as certain programs "prefer" it
    sed -i 's/^#en_GB.UTF-8 UTF-8/en_GB.UTF-8 UTF-8/' /etc/locale.gen
    sed -i 's/^#en_US.UTF-8 UTF-8/en_US.UTF-8 UTF-8/' /etc/locale.gen
    locale-gen
    pacman -Sy --noconfirm hunspell-en_gb
    ln -sf /usr/share/zoneinfo/Europe/London /etc/localtime
    hwclock --systohc
    echo LANG=en_GB.UTF-8 > /etc/locale.conf
    echo "KEYMAP=uk" > /etc/vconsole.conf
    # Set up hosts
    echo archlinux > /etc/hostname
    echo -e "\n127.0.0.1	localhost\n::1			localhost\n127.0.0.1	archlinux" >> /etc/hosts
    # Set Nano as the default terminal text editor
    echo -e "\n# Set terminal editor to Nano\nEDITOR=nano" >> /etc/environment
    # Install the mainline kernel
    pacman -S linux linux-headers linux-firmware --needed --noconfirm
    # Enable networking
	systemctl enable NetworkManager
    # Install the bootloader
	grub-install --target=x86_64-efi --efi-directory=/efi --bootloader-id=GRUB
	grub-mkconfig -o /boot/grub/grub.cfg
    # Set up the initial user account
	useradd -mG power,storage,wheel $username
	echo "$username:$password" | chpasswd
    # Disable logging in as root and give the initial user sudo privledges 
	passwd --lock root
	sed -i 's/^# %wheel ALL=(ALL:ALL) ALL/%wheel ALL=(ALL:ALL) ALL/' /etc/sudoers
    # Install some useful utilities
	pacman -S --noconfirm ufw clamav hblock
    # Configure ClamAV
	freshclam
	systemctl enable clamav-freshclam.service
	systemctl enable clamav-daemon.service
    # Configure UFW
	ufw limit 22/tcp
	ufw allow 80/tcp
	ufw allow 443/tcp
	ufw default deny incoming
	ufw default allow outgoing
	ufw limit ssh
	ufw enable
	systemctl enable ufw.service
    # Run hblock
	hblock
    # Prompt the user to reboot
	echo "Install complete, please reboot"
EOF
else
    echo "Stopping installation"
fi